import es, urllib, time, path, popuplib, playerlib, socket
import errorlib

# Addon Informations
info = es.AddonInfo()
info.name     = "Download URL Manager"
info.version = "1.3"
info.url      = "http://addons.eventscripts.com/addons/view/download_manager"
info.author   = "Ojii"
info.basename = "download_manager"
es.ServerVar('dlm_version', info['version'], 'Download Manager Addon by Ojii').makepublic()

# Some globals
gamepath = path.path(es.ServerVar('eventscripts_gamedir'))
cfgpath = gamepath.joinpath('cfg/download_manager.txt')
adminpath = gamepath.joinpath('cfg/download_manager_admins.txt')
admins = []
em = {}
method = 'ping'
ver = es.ServerVar('download_manager_ver',info.version,'DLM version')
ver.makepublic()
timeout = es.ServerVar('dlm_timeout',5,'DLM timeout (in seconds)')
timeout.makepublic()
method = es.ServerVar('dlm_method','ping','DLM method (ping or position)')
method.makepublic()

# Register all needed commands and set the download url
def load():
    errorlib.registerAddon(info,'me@ojii.ch')
    set_sv_downloadurl()
    if not es.exists('saycommad','!downloadurl'):
        es.regsaycmd('!downloadurl','download_manager/displayurl')
    if not es.exists('saycommand','!downloadmanager'):
        es.regsaycmd('!downloadmanager','download_manager/manager')
    if not es.exists('command','downloadmanager'):
        es.regcmd('downloadmanager','download_manager/manager_rcon')
    if not es.exists('clientcommand','downloadurlmanagerinputboxchoice'):
        es.regclientcmd('downloadurlmanagerinputboxchoice','download_manager/inputboxchoice')

# unregister the saycommands
def unload():    
    errrorlib.unregisterAddon('download_manager')
    es.unregsaycmd('!downloadurl')
    es.unregsaycmd('!downloadmanager')

# On map start: set the download url
def es_map_start(e):
    set_sv_downloadurl()

# This function checks both for connection to the server and map availability on the server
def connection(server,map):
    if not server.endswith('/'):
        server += '/'
    try:
        urllib.urlopen(server + map + '.bsp')
        return True
    except:
        return False

# Core function of this script, sets the correct download url
def set_sv_downloadurl():
    socket.setdefaulttimeout(int(es.ServerVar('dlm_timeout')))
    if cfgpath.isfile():
        currentmap = es.ServerVar('eventscripts_currentmap')
        lines = cfgpath.lines(retain=False)
        pinglist = []
        for link in lines:
            if connection(link,currentmap):
                if method == 'position':
                    es.server.queuecmd('sv_downloadurl "%s"' % link.strip('"'))
                    es.server.queuecmd('sv_allowdownload 1')
                    es.server.queuecmd('sv_allowupload 1')
                    break
                else:
                    pinglist.append([ping(link),link])
            else:
                es.dbgmsg(1,'[DLM] Could not establish a connection to %s or the map %s could not be found' % (link,currentmap))
        pinglist.sort()
        if not pinglist == []:
            es.server.queuecmd('sv_downloadurl "%s"' % pinglist[0][1])
        else:
            es.dbgmsg(0,'[DLM] Could not connect to any server in the config files')
            es.server.queuecmd('sv_downloadurl ""')
    else:
        es.dbgmsg(0,'[DLM] Could not find file %s' % cfgpath)
        es.server.queuecmd('sv_downloadurl ""')

# This function reads the admin file
def get_admins():
    global admins
    if adminpath.isfile():
        admins = adminpath.lines(retain=False)
    else:
        es.dbgmsg(0,'[DLM] Could not find file %s' % adminpath)

# This function displays the url to admins
def displayurl():
    get_admins()
    if es.getplayersteamid(es.getcmduserid()) in admins:
        es.tell(es.getcmduserid(),'The download URL is currently ' + es.ServerVar('sv_downloadurl'))

# Opens the ingame admin menu
def manager():
    global em
    get_admins()
    userid = es.getcmduserid()
    if es.getplayersteamid(userid) in admins:
        em[userid] = {'position':'main','link':None}
        es.escmenu(30,userid,"DownloadURL Manager","DownloadURL Manager Ingame Configurater","Add Server","Remove Server","View Servers","Add Admin","Remove Admin","View Admins","Change Method")
    else:
        es.tell(userid,"You are not an admin")

# Displays either the servers or the admins
def disp(method,userid):
    s = ''
    if method == 'admins':
        for admin in admins:
            s += admin + '\n'
        s = s[:-1]
    elif method == 'servers':
        if cfgpath.isfile():
            s = cfgpath.text()
    else:
        return
    es.esctextbox(30, userid,"DownloadURL Manager %s" % method, s)

# Handles escmenu choices
def es_client_command(ev):
    global lines, em, method
    get_admins()
    if ev['command'] == 'menuselect' and ev['commandstring'].isdigit():
        choice = int(ev['commandstring'])
        userid = int(ev['userid'])
        if em.has_key(userid):
            if em[userid]['position'] == 'main':
                if choice == 1:
                    es.escinputbox(30,userid,"DownloadURL Manager","Enter the link to the server please:","downloadurlmanagerinputboxchoice addserver")
                elif choice == 2:
                    em[userid]['position'] = 'rmvsrvr'
                    if cfgpath.isfile():
                        lines = cfgpath.lines(retain=False)
                        rmvspopup = popuplib.easymenu('remove_server_popup','_popup_choice',popup_choice)
                        rmvspopup.settitle('Choose a server to remove from the list:')
                        rmvspopup.setdescription('')
                        rmvspopup.visualstyle = 2
                        for link in lines:
                            rmvspopup.addoption(link,link)
                        rmvspopup.send(userid)
                elif choice == 3:
                    disp('servers',userid)
                elif choice == 4:
                    es.escinputbox(30,userid,"DownloadURL Manager","Enter a SteamID to add to the admins please:","downloadurlmanagerinputboxchoice addadmin")
                elif choice == 5:
                    em[userid]['position'] = 'rmvadmn'
                    namedict = {}
                    for userid in playerlib.getUseridList('#human'):
                        s = es.getplayersteamid(userid)
                        n = es.getplayername(userid)
                        namedict[s] = n
                    rmvapopup = popuplib.easymenu('remove_admin_popup','_popup_choice',popup_choice)
                    rmvapopup.settitle('Choose an admin to remove from the list:')
                    rmvapopup.visualstyle = 2
                    rmvapopup.setdescription('Admins with a name in brackets are currently on the server')
                    for admin in admins:
                        if namedict.has_key(admin):
                            rmvapopup.addoption(admin,"%s (%s)" % (s,namedict[s]))
                        else:
                            rmvapopup.addoption(admin,admin)
                    rmvapopup.send(userid)
                elif choice == 6:
                    disp('admins',userid)
                elif choice == 7:
                    em[userid]['position'] = 'chngmthd'
                    es.escmenu(30,userid,"DownloadURL Manager","Choose a method for server selection (current: %s)" % method,"Ping","Position")
            elif em[userid]['position'] == 'srvrpstn':
                choice -= 1
                lines.insert(choice,em[userid]['link'])
                cfgpath.write_lines(lines)
            elif em[userid]['position'] == 'chngmthd':
                if choice == 1:
                    method = 'ping'
                elif choice == 2:
                    method = 'position'

# Handles popup choices
def popup_choice(userid,choice,menuname):
    global admins
    if em[userid]['position'] == 'rmvadmn':
        admins.remove(choice)
        adminpath.write_lines(adminpath)
    elif em[userid]['position'] == 'rmvsrvr':
        f = open(cfgpath,'r')
        lines = f.read().splitlines()
        f.close()
        lines.remove(choice)
        cfgpath.write_lines(lines)
    elif em[userid]['position'] == 'srvrpstn':
        f = open(cfgpath,'r')
        lines = f.read().splitlines()
        f.close()
        lines.insert(choice,em[userid]['link'])
        cfgpath.write_lines(lines)

# Handles inputbox choices
def inputboxchoice():
    global em
    if es.getargv(1) == 'addserver':
        em[e['userid']]['position'] = 'srvrpstn'
        em[e['userid']]['link'] = es.getargv(2)
        posmenu = popuplib.easymenu('position_menu','_popup_choice',popup_choice)
        posmenu.settitle("Choose a position to insert the server (above the choosen link):")
        posmenu.visualstyle = 2
        lines = cfgpath.lines(retain=False)
        for pos in range(len(lines)):
            posmenu.addoption(pos,lines[pos])
        posmenu.send(e['userid'])
    elif es.getargv(1) == 'addadmin':
        sid = convert_steamid(es.getargv(2))
        admins.append(sid)
        adminpath.write_lines(adminpath)

# Rcon admin
def manager_rcon():
    global admins, method
    get_admins()
    if cfgpath.isfile():
        lines = cfgpath.lines(retain=False)
        if es.getargv(1) == 'addurl':
            if es.getargc() == 4:
                pos = int(es.getargv(2)) - 1
                if pos > len(lines):
                    lines.append(es.getargv(3))
                else:
                    lines.insert(pos,es.getargv(3))
            else:
                lines.insert(0,es.getargv(2))
            cfgpath.write_lines(lines)
        elif es.getargv(1) == 'delurl':
            if es.getargv(2) in lines:
                lines.remove(es.getargv(2))
                cfgpath.write_lines(lines)
        elif es.getargv(1) == 'dispurl':
                es.dbgmsg(0,'')
                es.dbgmsg(0,'[DLM] DownloadURL Manager Servers:')
                es.dbgmsg(0,'[DLM] ++++++++++++++++++++++++++++')
                for line in lines:
                    es.dbgmsg(0,line)
                es.dbgmsg(0,'')
    if es.getargv(1) == 'addadmin':
        newadmin = convert_steamid(es.getargv(2))
        if newadmin:
            admins.append(newadmin)
            adminpath.write_lines(adminpath)
    elif es.getargv(1) == 'deladmin':
        oldadmin = convert_steamid(es.getargv(2))
        if oldadmin in admins:
            admins.remove(oldadmin)
            adminpath.write_lines(adminpath)
    elif es.getargv(1) == 'dispadmin':
            es.dbgmsg(0,'')
            es.dbgmsg(0,'[DLM] DownloadURL Manager Admins:')
            es.dbgmsg(0,'[DLM] +++++++++++++++++++++++++++')
            for admin in admins:
                es.dbgmsg(0,admin)
            es.dbgmsg(0,'')
    elif es.getargv(1) == 'dispmethod':
            es.dbgmsg(0,'')
            es.dbgmsg(0,'[DLM] DownloadURL Manager Method: %s' % method)
            es.dbgmsg(0,'')
    elif es.getargv(1) == 'method':
        if es.getargc() == 3:
            if es.getargv(2) in ['ping','position']:
                method = es.getargv(2)
            elif method == 'ping':
                method = 'position'
            else:
                method = 'ping'
    elif es.getargc() == 1:
        es.dbgmsg(0,'')
        es.dbgmsg(0,'[DLM] Use one of the following arguments for the downloadmanager commend:')
        es.dbgmsg(0,'  addurl <url> (adds an url to the serverlist)')
        es.dbgmsg(0,'  delurl <url> (removes an url from the serverlist)')
        es.dbgmsg(0,'  dispurl (display the content of the serverlist)')
        es.dbgmsg(0,'  addadmin <identifier> (adds an admin to the adminlist, identifier must be unique (steamid,userid,name))')
        es.dbgmsg(0,'  deladmin <identifier> (removes an admin from the adminlist, identifier must be unique)')
        es.dbgmsg(0,'  dispadmin (displays a list of admins)')
        es.dbgmsg(0,'  method [<method>] (changes the method or sets it to the choosen method (ping or position))')
        es.dbgmsg(0,'  dispmethod (displays the current method (ping or position))')
        es.dbgmsg(0,'')

# Converts wrong SteamIDs to correct format
def convert_steamid(s):
    if s.startswith('STEAM_') and s.count(':') == 2:
        return s
    elif s.count(':') == 2:
        return 'STEAM_' + s
    elif s.isdigit():
        return es.getplayersteamid(int(s))
    else:
        thelist = []
        for player in playerlib.getUseridList('#human'):
            if es.getplayername(player).startswith(s):
                thelist.append(es.getplayersteamid(player))
        if len(thelist) == 1:
            return thelist[0]
        else:
            return None

# Measures the latency to a server
def ping(link):
    start = time.time()
    u = urllib.urlopen(link)
    return time.time() - start